'use strict'

const css = require('css'),
      object = require('@mightyplow/jslib').object,
      stylesheets = require('@mightyplow/jslib').stylesheet

const mediaProp = object.prop('media'),
      rulesProp = object.prop('rules'),
      isMediaRule = stylesheets.isRuleType('media')

const allMediaKey = 'all'

const stringifyCss = options => stylesheet => css.stringify(stylesheet, options)

const createResult = (media, noMedia) => {
    media = media || {}

    if (noMedia) {
        if (media[allMediaKey] !== undefined) {
            media[allMediaKey] = media[allMediaKey].concat(noMedia)
        } else {
            media[allMediaKey] = noMedia
        }
    }

    return media
}

module.exports = (input, options) => {
    options = options || {}

    object.defaults(options, {
        minRules: 20,
        compress: true,
        asAst: false,
        source: undefined
    })

    // throw an error when input is malformed css
    const inputStylesheet = css.parse(input, {
        silent: false,
        source: options.source
    })

    const rules = stylesheets.getRules(inputStylesheet)
    const medias = rules.filter(isMediaRule)

    // find all media query rules and combine them per selector
    const mediaBundles = medias.reduce((acc, media) => {
        const mediaSelector = mediaProp(media)

        if (acc[mediaSelector] === undefined) {
            acc[mediaSelector] = []
        }

        acc[mediaSelector].push.apply(acc[mediaSelector], rulesProp(media))

        return acc
    }, {})

    // filter bundles which don't match the min rule setting
    const filteredBundles = Object.keys(mediaBundles).reduce((filtered, key) => {
        const rules = mediaBundles[key]

        if (rules.length >= options.minRules) {
            filtered[key] = rules
        }

        return filtered
    }, {})

    /*
     Get the rules, which are not contained in the filtered bundles.
     The criterias for not being in matches is that it's not a media rule and that the selector of the media rule
     is not in the bundles
     */
    const unmatchedRules = rules.filter(rule => {
        return !isMediaRule(rule) || Object.keys(filteredBundles).indexOf(mediaProp(rule)) < 0;
    })

    // pack the rules into stylesheet objects
    const mediaStylesheet = Object.keys(filteredBundles).reduce((filtered, key) => {
        filtered[key] = stylesheets.createStylesheetAst(filteredBundles[key])
        return filtered
    }, {})

    const noMediaStylesheet = stylesheets.createStylesheetAst(unmatchedRules)

    // create stringify function with given options
    const stringifyStylesheet = stringifyCss({
        compress: options.compress
    })

    // create css strings for the stylesheets
    const mediaCssStrings = Object.keys(mediaStylesheet).reduce((cssStrings, key) => {
        cssStrings[key] = stringifyStylesheet(mediaStylesheet[key])
        return cssStrings
    }, {})

    const noMediaCssString = stringifyStylesheet(noMediaStylesheet);

    // when css strings should be returned, we're done
    if (options.asAst === false) {
        return createResult(mediaCssStrings, noMediaCssString)
    }


    // when ASTs should be returned, the css strings get recompiled to ensure proper start and end values
    const mediaAsts = Object.keys(mediaCssStrings).reduce((asts, key) => {
        asts[key] = css.parse(mediaCssStrings[key])
        return asts
    }, {})

    return createResult(mediaAsts, css.parse(noMediaCssString))
}